#! /bin/sh -posix
# audit v0.7

#-----------------------------------------------------------------------
# Global variables
#-----------------------------------------------------------------------
# The version number of the audit script
VER="v0.7"

# The name of the NEW audit file.
NEW="/root/audit-current"

# The name of the OLD audit file.
OLD="/root/audit-old"

# a simple lock file for our menu.
MENU="/tmp/.audit.tmp"

# A default set of directories to be scanned.
DEFAULT="/bin /etc /lib /sbin /usr/bin /usr/sbin /usr/local/bin"

# The name of the audit report.
RPT="/root/.audit.rpt"

#-----------------------------------------------------------------------
# Make sure all external dependencies are satisfied
#-----------------------------------------------------------------------
TROUBLE() {
clear
cat <<MSG

audit${VER} digs <mjs@nts-online.net>

External dependencies remain UN-satisfied aborting...
audit depends on the following tools to operate properly:

cat
cp
date
dialog
diff
find
gzip
kill
md5sum
mv
pidof
rm
tar
MSG
exit 1
}

if ! which \
{cat,cp,date,dialog,diff,find,gzip,kill,md5sum,mv,pidof,rm,tar} >/dev/null 2>&1; then
   ERROR="TROUBLE"
fi

# if the string $ERROR is greater than zero, we abort.
if [ ${#ERROR} -ne 0 ]; then
${ERROR}
fi

#-----------------------------------------------------------------------
UBER_MAN() {
#-----------------------------------------------------------------------
# Allow root only to run this script otherwise abort.
clear
cat <<MSG

Sorry - only root can run this script...
MSG
exit 1
}

if [ ${UID} -ne 0 ]; then UBER_MAN; fi

#-----------------------------------------------------------------------
VERIFY() {
#-----------------------------------------------------------------------
# verify that our resource file is located in /root & source it.
if [ -e /root/.auditrc ]; then
. /root/.auditrc
fi
}

#-----------------------------------------------------------------------
ABOUT() {
#-----------------------------------------------------------------------
cat <<MSG >$MENU

AUDIT HELP:

Exit ...................................................... [e]
Scroll .................................................... [arrow keys]
Search .................................................... [/]
Repeat last search ........................................ [n]    

Q. What is audit?
   
A. audit is a script that will attempt to record any changes to files
on your computer by generating a unique checksum for each file in the
directories scanned, it was built with simplicity and ease of use in
mind. Out of the box, audit is tuned to run with a reasonable set of
defaults in place. If you'd like a way to keep an eye on your files
this proves to be an ideal way to accomplish this with a minimum of
fuss.
   
Q. How can audit help me?
   
A. By tracking any changes (including binary) on your system; the
addition and/or deletion of files. audit can accurately pinpoint the
slightest change including any monkey-business on your Windows/DOS
partitions. If Linux can mount it, audit can scan it.
   
Q. Can audit clean my system of virii?
   
A. NO - audit is not a virus detection scheme, if you feel your
system has been compromised in some way, audit can assist you in
locating files that may have been tampered with, optionally you
can print the results of an audit for later comparison.
   
Q. Okay, how do I set it up?
   
A. NOTE: [These instructions are also included with the archive.]
   
1. First you'll need to download the audit package located here:

   http://console-newsletter.hypermart.net/audit/index.html
   
2. Now unpack the tar and gzipped file to your /tmp folder:
   
   mv audit.tgz /tmp
   tar zxf audit.tgz
   
3. Working as root for the remaining steps, cd to /tmp/audit/dialog
and compile and install the included dialog app:

NOTE: [The script will fail if you use an earlier version of dialog,
best to use the one included]
   
   make
   make install
   
4. move audit to a folder on your path (/bin is a good choice):
   
   mv /tmp/audit/audit /bin
   
5. Make sure the script is executable:
   
   chmod +x /bin/audit
   
In an effort to keep the script simple I've opted to use only
readily available tools common to every Linux distro, they are:

cat cp date dialog diff find gzip kill md5sum mv pidof rm tar

At this point you should be ready to run your first audit.
From the command line type:
   
   audit
   
Using your arrow keys scroll up/down to a desired menu entry then
press Enter. Please take a moment to read the help first as several
tips are included to ensure successful use. During your next step
scroll down to [SETUP] and press enter.

Just enter the directories you wish to have scanned in quotes with
each directory separated by a space like so:
   
   "/bin /sbin /home/myfiles"

Or to scan everything mounted just enter:

   "/"

NOTE: ["/" will include things like any currently mounted CDROMS too!]
   
Confirm your choice and at the main panel highlight [AUDIT] and tap
Enter, and you'll see audit begin working. If you chose alot of dir-
ectories to be scanned, go grab your self a soda and take a five mi-
nute break while audit does it's work, when you return you'll see the
next panel which reports any changes (or not) to your files.

NOTE: [The first time you perform an audit the list may be long.]

The remaining menu entries are self explanatory and easy to understand.
audit shouldn't be viewed as any type of recovery tool but rather as an
adjunct to allow you to know what's going on with your files. With this
understanding audit will give you (I hope) good service.
   
Q. GUI's are for the weak, can't I run audit without the interface?
   
A. Sure, at the command line invoke:
   
   audit -h
   
Here are the flags audit understands:
   
Usage: audit -h -s -q
   
-h This help screen
-s Begin scanning without delay
-q Quiet, as option [-s] but without the GUI
   
(The -q flag works great when launched from your cron-tab.)
   
Q. How much does audit cost?
   
A. Nothing - Its 100% free.

This work is governed under the GNU license which means if you find
a way to make it better then you're effort will not only make for a
better tool on your boxen but others as well. If you have any sugg-
estions for improving audit, drop me a quick note.
   
later on,
digs <mjs@nts-online.net>
   
MSG
dialog --title "audit $VER HELP" --no-shadow --textbox "$MENU" 0 0
}

#-----------------------------------------------------------------------
HELP() {
#-----------------------------------------------------------------------
clear
cat <<MSG
audit

A simple reliable way to detect changes in your system files.

Usage: audit -h -s -q

-h This help screen
-s Begin the audit without further delay
-q Quiet, as option [-s] but without the GUI

MSG
exit 0
}

#-----------------------------------------------------------------------
NODIRS() {
#-----------------------------------------------------------------------
dialog --title "audit confirm" --yesno "\n\
Since you did NOT choose any directoreis to be scanned\n\
the following defaults will be used instead:\n\n\
${DEFAULT}\n\n\
Is this correct?" 0 0

case $? in
  0)return 0;;
  1)SETUP;;
  255)#ESC pressed
      return 0;;
esac
}

#-----------------------------------------------------------------------
YESDIRS() {
#-----------------------------------------------------------------------
dialog --title "audit confirm" --yesno " \
The following directories will be scanned:\n\n\
${DIRS:=${DEFAULT}}\n\n\
Is this correct?" 0 0

case $? in
  0)return 0;;
  1)SETUP;;
  255)#ESC pressed
      return 0;;
esac
}

#-----------------------------------------------------------------------
CONFIRM() {
#-----------------------------------------------------------------------
VERIFY
if [ ${#DIRS} -eq 0 ]; then
   NODIRS
else
   YESDIRS
fi
}

#-----------------------------------------------------------------------
SETUP() {
#-----------------------------------------------------------------------
dialog --title "audit SETUP" --inputbox \
"  Type the complete path to any extra directories you wish to
have scanned in quotes with each directory separated by a space.\n\n\
               Example: \"/bin /sbin /home/jenny\"" 0 0 2> $MENU

case $? in
  0)#prepare default ./.auditrc
    if [ ${#DIRS} -eq 0 ]; then 
    DIRS="${DEFAULT}"
    else DIRS="${DIRS}"
    fi; echo "DIRS=`cat $MENU`" >./.auditrc && CONFIRM;;
    1)#Cancel pressed
      return 0;;
  255)#ESC pressed
      return 0;;
esac
}

#-----------------------------------------------------------------------
PRINT() {
#-----------------------------------------------------------------------
if ! [ -e ${RPT} ]; then
   dialog --title "ERROR Cant find ${RPT}" --msgbox "\n\
Cant [PRINT] the audit report, please\n\
run [AUDIT] from the main menu first.\n\
" 0 0
else
dialog --title "Print ${RPT}" --yesno "\n\
Press [YES] to Print or [NO] to cancel.\n\
" 0 0
case $? in
  0)lpr ${RPT} &;;
  1);;
  255)#ESC pressed
      ;;
esac
return 0
fi
}

#-----------------------------------------------------------------------
VIEW() {
#-----------------------------------------------------------------------
if ! [ -e ${RPT} ]; then
   dialog --title "ERROR Cant find ${RPT}" --msgbox "\n\
Cant [VIEW] the audit report, please\n\
run [AUDIT] from the main menu first.\n\
" 0 0
else 
   dialog --title "audit scan results" --textbox "${RPT}" 0 0
fi
}

#-----------------------------------------------------------------------
MSGOK() {
#-----------------------------------------------------------------------
# The message displayed when there are NO changes found.
cat <<GOOD >${RPT}

Host:${HOSTNAME}
Date:`date +%D`
Type:md5sum
Scan:
${DIRS:=${DEFAULT}}

NO differing checksums were detected.

GOOD
}

#-----------------------------------------------------------------------
MSG_DIFF() {
#-----------------------------------------------------------------------
# The message displayed when there ARE changes found.
cat <<CHANGES >${RPT}

Host:${HOSTNAME}
Date:`date +%D`
Type:md5sum
Scan:
${DIRS:=${DEFAULT}}

Differing checksums were detected for:

`diff ${OLD} ${NEW}`
CHANGES
}

#-----------------------------------------------------------------------
TAILBOX() {
#-----------------------------------------------------------------------
dialog --title "Please wait while audit scans your directories..." \
--tailbox "${NEW}" 20 70
case $? in
    0)return 0;;
  255)return 0;;
esac
}

#-----------------------------------------------------------------------
SCAN() {
#-----------------------------------------------------------------------
# This routine performs the scan

# if the current audit is compressed then extract it,
# and move it to OLD
if [ -e ${NEW}.gz ]; then
   gzip -fd ${NEW}.gz && mv ${NEW} ${OLD}
fi

# create our new current audit file
cd /
cat <<EOF >${NEW}

md5sum audit control file.
`date +%D`
${HOSTNAME}

EOF

# create md5sums for all files listed in DIRS while generating a tailbox.
find ${DIRS:=${DEFAULT}} -path /proc -prune -o -type f -exec md5sum -b {} \; >>${NEW} && \
kill -1 `pidof dialog` & TAILBOX

# if OLD exists then diff OLD and NEW
( if [ -e ${OLD} ]; then
    eval diff ${OLD} ${NEW} >/dev/null && MSGOK || MSG_DIFF
  fi
# delete OLD because we no longer need it.
  if [ -e ${OLD} ]; then
    rm ${OLD}
  else
    cp ${NEW} ${RPT}  
  fi
# Compress NEW
  if [ -e ${NEW} ]; then
    gzip -f9 ${NEW}
  fi )      
}

#-----------------------------------------------------------------------
MAIN() {
#-----------------------------------------------------------------------
dialog --clear --title "audit ${VER}" --menu "\n\
 This script will create an audit control file with checksums\n\
 of all the files in the directories you choose during [SETUP].\n\
If any changes are detected then; you'll have a chance to [VIEW]\n\
   and/or [PRINT] a listing of those files that have changed.\n\n\
         Scroll up/down using your arrow keys then tap\n\
                  Enter on a highlighted entry.\n\
" 0 0 6 \
"HELP " "               How to use this tool               " \
"SETUP" "            Define your configuration             " \
"AUDIT" "               Perform an audit now               " \
"PRINT" "             Print your latest audit              " \
"VIEW " "         View your previous audit results         " \
"EXIT " "                  Leave this menu                 " 2> $MENU

SELECT="`cat $MENU`"

case ${SELECT} in
  "HELP ")ABOUT && MAIN;;
  "SETUP")SETUP && MAIN;;
  "AUDIT")SCAN  && VIEW && MAIN;;
  "PRINT")PRINT && MAIN;;
  "VIEW ")VIEW  && MAIN;;
  "EXIT ");;
      255)#ERROR;;
esac
reset
clear
exit 0
}

#-----------------------------------------------------------------------
# The entry point for execution begins here.
#-----------------------------------------------------------------------
VERIFY
case $1 in
   -[h/H])HELP;;
   -[s/S])SCAN && VIEW;;
   -[q/Q])SCAN >/dev/null 2>&1;;
        *)MAIN;;
esac
reset
if [ -e ${MENU} ]; then rm ${MENU}; fi 
exit 0

